@@ -36,13 +36,13 @@
 #define TEGRA_GPIO_BACKLIGHT		TEGRA_GPIO_PD4
 #define TEGRA_GPIO_LVDS_SHUTDOWN	TEGRA_GPIO_PB2
 #define TEGRA_GPIO_BACKLIGHT_PWM	TEGRA_GPIO_PU5
-#define SEABOARD_GPIO_BACKLIGHT_VDD	TEGRA_GPIO_PW0
-#define ASYMPTOTE_GPIO_BACKLIGHT_VDD	TEGRA_GPIO_PU5
+#define TEGRA_GPIO_BACKLIGHT_VDD	TEGRA_GPIO_PW0
 #define TEGRA_GPIO_EN_VDD_PNL		TEGRA_GPIO_PC6
 #define TEGRA_GPIO_MAGNETOMETER		TEGRA_GPIO_PN5
 #define TEGRA_GPIO_NCT1008_THERM2_IRQ	TEGRA_GPIO_PN6
 #define TEGRA_GPIO_HDMI_HPD		TEGRA_GPIO_PN7
 #define TEGRA_GPIO_ISL29018_IRQ		TEGRA_GPIO_PZ2
+#define ASYMPTOTE_GPIO_TSL2563_IRQ		TEGRA_GPIO_PZ2
 #define TEGRA_GPIO_MPU3050_IRQ		TEGRA_GPIO_PZ4
 #define TEGRA_GPIO_AC_ONLINE		TEGRA_GPIO_PV3
 #define TEGRA_GPIO_BATT_DETECT		TEGRA_GPIO_PP2
