@@ -199,6 +199,9 @@ struct nvec_chip {
 extern int nvec_write_async(struct nvec_chip *nvec, const unsigned char *data,
 			     short size);
 
+extern struct nvec_msg *nvec_write_sync(struct nvec_chip *nvec,
+					const unsigned char *data, short size);
+
 extern int nvec_register_notifier(struct nvec_chip *nvec,
 				  struct notifier_block *nb,
 				  unsigned int events);
