@@ -285,7 +285,7 @@ out_unlock:
  *	trouble so this is our best effort to reboot.  This is
  *	safe to call in interrupt context.
  */
-void __weak emergency_restart(void)	/* defer to kernel/preserved.c */
+void emergency_restart(void)
 {
 	kmsg_dump(KMSG_DUMP_EMERG);
 	machine_emergency_restart();
